/** Events are **regular**, which means they're all expected to be stored by relays. */
export declare function isRegularKind(kind: number): boolean;
/** Events are **replaceable**, which means that, for each combination of `pubkey` and `kind`, only the latest event is expected to (SHOULD) be stored by relays, older versions are expected to be discarded. */
export declare function isReplaceableKind(kind: number): boolean;
/** Events are **ephemeral**, which means they are not expected to be stored by relays. */
export declare function isEphemeralKind(kind: number): boolean;
/** Events are **parameterized replaceable**, which means that, for each combination of `pubkey`, `kind` and the `d` tag, only the latest event is expected to be stored by relays, older versions are expected to be discarded. */
export declare function isParameterizedReplaceableKind(kind: number): boolean;
/** Classification of the event kind. */
export type KindClassification = 'regular' | 'replaceable' | 'ephemeral' | 'parameterized' | 'unknown';
/** Determine the classification of this kind of event if known, or `unknown`. */
export declare function classifyKind(kind: number): KindClassification;
export declare const Metadata = 0;
export declare const ShortTextNote = 1;
export declare const RecommendRelay = 2;
export declare const Contacts = 3;
export declare const EncryptedDirectMessage = 4;
export declare const EncryptedDirectMessages = 4;
export declare const EventDeletion = 5;
export declare const Repost = 6;
export declare const Reaction = 7;
export declare const BadgeAward = 8;
export declare const GenericRepost = 16;
export declare const ChannelCreation = 40;
export declare const ChannelMetadata = 41;
export declare const ChannelMessage = 42;
export declare const ChannelHideMessage = 43;
export declare const ChannelMuteUser = 44;
export declare const OpenTimestamps = 1040;
export declare const FileMetadata = 1063;
export declare const LiveChatMessage = 1311;
export declare const ProblemTracker = 1971;
export declare const Report = 1984;
export declare const Reporting = 1984;
export declare const Label = 1985;
export declare const CommunityPostApproval = 4550;
export declare const JobRequest = 5999;
export declare const JobResult = 6999;
export declare const JobFeedback = 7000;
export declare const ZapGoal = 9041;
export declare const ZapRequest = 9734;
export declare const Zap = 9735;
export declare const Highlights = 9802;
export declare const Mutelist = 10000;
export declare const Pinlist = 10001;
export declare const RelayList = 10002;
export declare const BookmarkList = 10003;
export declare const CommunitiesList = 10004;
export declare const PublicChatsList = 10005;
export declare const BlockedRelaysList = 10006;
export declare const SearchRelaysList = 10007;
export declare const InterestsList = 10015;
export declare const UserEmojiList = 10030;
export declare const FileServerPreference = 10096;
export declare const NWCWalletInfo = 13194;
export declare const LightningPubRPC = 21000;
export declare const ClientAuth = 22242;
export declare const NWCWalletRequest = 23194;
export declare const NWCWalletResponse = 23195;
export declare const NostrConnect = 24133;
export declare const HTTPAuth = 27235;
export declare const Followsets = 30000;
export declare const Genericlists = 30001;
export declare const Relaysets = 30002;
export declare const Bookmarksets = 30003;
export declare const Curationsets = 30004;
export declare const ProfileBadges = 30008;
export declare const BadgeDefinition = 30009;
export declare const Interestsets = 30015;
export declare const CreateOrUpdateStall = 30017;
export declare const CreateOrUpdateProduct = 30018;
export declare const LongFormArticle = 30023;
export declare const DraftLong = 30024;
export declare const Emojisets = 30030;
export declare const Application = 30078;
export declare const LiveEvent = 30311;
export declare const UserStatuses = 30315;
export declare const ClassifiedListing = 30402;
export declare const DraftClassifiedListing = 30403;
export declare const Date = 31922;
export declare const Time = 31923;
export declare const Calendar = 31924;
export declare const CalendarEventRSVP = 31925;
export declare const Handlerrecommendation = 31989;
export declare const Handlerinformation = 31990;
export declare const CommunityDefinition = 34550;
